﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;
using Infragistics.Controls.Charts;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Diagnostics;
using Infragitstics.MotionFramework.MotionFramework;

namespace Infragitstics.MotionFramework
{
    public class CoreMF
    {
        #region fields
        private DispatcherTimer _timer;
        private int _currentElementShown;
        private int _maxElementsCount = 0;
        #endregion

        #region ctor
        public CoreMF()
        {
            this._timer = new DispatcherTimer();
            this._timer.Interval = TimeSpan.FromMilliseconds(500);
            this._timer.Tick += new EventHandler(Timer_Tick);
        }
        #endregion

        #region Properties

        #region DataChart
        private XamDataChart _dataChart;
        public XamDataChart DataChart
        {
            get { return _dataChart; }
            set { _dataChart = value; }
        }
        #endregion

        #region Legend
        private Legend _chartLegend;

        public Legend ChartLegend
        {
            get { return _chartLegend; }
            set { _chartLegend = value; }
        }
        #endregion

        #region DataSource
        private Dictionary<int, IList> _dataSource;
        public Dictionary<int, IList> DataSource
        {
            get { return _dataSource; }
            set { _dataSource = value; }
        }
        #endregion

        #region XAxisName
        private string _xAxisName;
        public string XAxisName
        {
            get { return _xAxisName; }
            set { _xAxisName = value; }
        }
        #endregion

        #region YAxisNAme
        private string _yAxisName;
        public string YAxisName
        {
            get { return _yAxisName; }
            set { _yAxisName = value; }
        }
        #endregion

        #region PropertyNameX
        private string _propertyNameX;
        public string PropertyNameX
        {
            get { return _propertyNameX; }
            set { _propertyNameX = value; }
        }
        #endregion

        #region PropertyNameY
        private string _propertyNameY;
        public string PropertyNameY
        {
            get { return _propertyNameY; }
            set { _propertyNameY = value; }
        }
        #endregion

        #region PropertyNameR
        private string _propertyNameR;
        public string PropertyNameR
        {
            get { return _propertyNameR; }
            set { _propertyNameR = value; }
        }
        #endregion

        #region MarkerTemplate
        private DataTemplate _markerTemplate;
        public DataTemplate MarkerTemplate
        {
            get { return _markerTemplate; }
            set { _markerTemplate = value; }
        }
        #endregion

        #region LegendItemTemplate
        private DataTemplate _legendItemTemplate;

        public DataTemplate LegendItemTemplate
        {
            get { return _legendItemTemplate; }
            set { _legendItemTemplate = value; }
        }
        #endregion

        #endregion

        #region Methods
        #region RequiredConditionsCheck
        private bool RequiredConditionsCheck()
        {
            bool ret = true;
            ret = (ret && this.DataChart != null);
            ret = (ret && this.DataSource != null);
            ret = (ret && this.XAxisName != null);
            ret = (ret && this.YAxisName != null);
            ret = (ret && this.PropertyNameX != null);
            ret = (ret && this.PropertyNameY != null);
            ret = (ret && this.PropertyNameR != null);
            return ret;
        }
        #endregion

        #region InitializeDataChart
        private void InitializeDataChart()
        {
            if (this.RequiredConditionsCheck())
            {
                this.DataChart.Series.Clear();
                this._currentElementShown = 0;
                this._maxElementsCount = this.DataSource[0].Count;
                foreach (var item in this.DataSource)
                {
                    this.InitScatterSeries(this.GenSeriesName(item.Key), item.Key);
                }
            }
        }
        #endregion

        #region GenSeriesName
        private string GenSeriesName(int key, bool isUnique = false)
        {
            if(isUnique)
                return string.Format("{0}|Series {0}", key);
            return string.Format("Series {0}", key);
        }

        #endregion
        
        #region ScatterSeriesHelper
        private void InitScatterSeries(string name, int dataOrder, IEnumerable dataSource = null)
        {
            ScatterSeries s = new ScatterSeries();
            s.Name = this.GenSeriesName(dataOrder, true);
            s.YMemberPath = this.PropertyNameY;
            s.XMemberPath = this.PropertyNameX;
            s.TransitionDuration = this._timer.Interval;
            s.Title = name;
            s.XAxis = this.DataChart.FindName(XAxisName) as NumericXAxis;
            s.YAxis = this.DataChart.FindName(YAxisName) as NumericYAxis;
            s.MarkerTemplate = this.MarkerTemplate;
            if (this.ChartLegend != null)
                s.Legend = this.ChartLegend;
            if (this.LegendItemTemplate != null)
                s.LegendItemTemplate = this.LegendItemTemplate;
            ItemsSource<DemoModel> ds = new ItemsSource<DemoModel>();
            ds.Add(new DemoModel { ValueX = 0, ValueY = 0, ValueR = 10, ToolTip = "" });
            s.ItemsSource = ds;
            if (dataSource != null)
                s.ItemsSource = dataSource;
            this.DataChart.Series.Add(s);
        }
        #endregion

        #region UpdateDataPoint
        private void UpdateDataPoint(Series series, DemoModel newDp)
        {
            ((IList<DemoModel>)series.ItemsSource)[0].ValueX = newDp.ValueX;
            ((IList<DemoModel>)series.ItemsSource)[0].ValueY = newDp.ValueY;
            ((IList<DemoModel>)series.ItemsSource)[0].ValueR = newDp.ValueR;
            ((IList<DemoModel>)series.ItemsSource)[0].ToolTip = newDp.ToolTip;
            ((ItemsSource<DemoModel>)series.ItemsSource).Refresh();
        }
        #endregion

        #region RebindDataChart()
        private void RebindDatachart()
        {
            foreach (var item in this.DataSource)
            {
                string SeriesName = this.GenSeriesName(item.Key, true);
                DemoModel newDp = (DemoModel)item.Value[this._currentElementShown];
                this.UpdateDataPoint(this.DataChart.Series[SeriesName], newDp);
            }
        }
        #endregion

        #region Play
        public void Play()
        {
            if (this.RequiredConditionsCheck())
            {
                this.InitializeDataChart();
                this._timer.Start();
            }
        }
        #endregion
        #endregion

        #region EventHandlers
        #region TimerTick
        void Timer_Tick(object sender, EventArgs e)
        {
            if (this._currentElementShown < this._maxElementsCount)
            {
                this.RebindDatachart();
                this._currentElementShown++;
            }
            else
            {
                this._timer.Stop();
                this._currentElementShown = 0;
            }
        }
        #endregion
        
        #endregion
    }
}
